@@ -2,12 +2,11 @@
 # this repository contains the full copyright notices and license terms.
 
 import copy
-import collections
 from functools import total_ordering
 
 from trytond.model import fields
 from trytond.error import WarningErrorMixin
-from trytond.pool import Pool, PoolBase
+from trytond.pool import Pool, PoolBase, PoolMeta
 from trytond.pyson import PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.url import URLMixin
@@ -16,8 +15,16 @@ from trytond.rpc import RPC
 __all__ = ['Model']
 
 
+class ModelMeta(PoolMeta):
+    @property
+    def __queue__(self):
+        pool = Pool()
+        Queue = pool.get('ir.queue')
+        return Queue.caller(self)
+
+
 @total_ordering
-class Model(WarningErrorMixin, URLMixin, PoolBase):
+class Model(WarningErrorMixin, URLMixin, PoolBase, metaclass=ModelMeta):
     """
     Define a model in Tryton.
     """
@@ -66,10 +73,10 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
 
         # Set _defaults
         cls._defaults = {}
-        fields_names = cls._fields.keys()
+        fields_names = list(cls._fields.keys())
         for field_name in fields_names:
             default_method = getattr(cls, 'default_%s' % field_name, False)
-            if isinstance(default_method, collections.Callable):
+            if callable(default_method):
                 cls._defaults[field_name] = default_method
 
         for k in cls._defaults:
@@ -78,7 +85,7 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
                 % (cls.__name__, k,)
 
         # Set name to fields
-        for name, field in cls._fields.iteritems():
+        for name, field in cls._fields.items():
             if field.name is None:
                 field.name = name
             else:
@@ -148,7 +155,7 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
                         value[field_name]).rec_name
 
         if not with_rec_name:
-            for field in value.keys():
+            for field in list(value.keys()):
                 if field.endswith('.rec_name'):
                     del value[field]
         return value
@@ -215,6 +222,8 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
                     'selection_change_with',
                     'domain',
                     'converter',
+                    'search_order',
+                    'search_context',
                     ):
                 if getattr(cls._fields[field], arg, None) is not None:
                     value = getattr(cls._fields[field], arg)
@@ -304,7 +313,7 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
             if res[field]['type'] == 'many2one':
                 target = cls._fields[field].get_target()
                 relation_fields = []
-                for target_name, target_field in target._fields.iteritems():
+                for target_name, target_field in target._fields.items():
                     if (target_field._type == 'one2many'
                             and target_field.model_name == cls.__name__
                             and target_field.field == field):
@@ -312,7 +321,7 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
                 # Set relation_field only if there is no ambiguity
                 if len(relation_fields) == 1:
                     res[field]['relation_field'], = relation_fields
-            if res[field]['type'] in ('datetime', 'time'):
+            if res[field]['type'] in ('datetime', 'time', 'timestamp'):
                 res[field]['format'] = copy.copy(cls._fields[field].format)
             if res[field]['type'] == 'selection':
                 res[field]['context'] = copy.copy(cls._fields[field].context)
@@ -330,11 +339,11 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
 
             # convert attributes into pyson
             for attr in ('states', 'domain', 'context', 'digits', 'size',
-                    'add_remove', 'format'):
+                    'add_remove', 'format', 'search_order', 'search_context'):
                 if attr in res[field]:
                     res[field][attr] = encoder.encode(res[field][attr])
 
-        for i in res.keys():
+        for i in list(res.keys()):
             # filter out fields which aren't in the fields_names list
             if fields_names:
                 if i not in fields_names:
@@ -354,7 +363,7 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
         if kwargs:
             self._values = {}
             parent_values = {}
-            for name, value in kwargs.iteritems():
+            for name, value in kwargs.items():
                 if not name.startswith('_parent_'):
                     setattr(self, name, value)
                 else:
@@ -372,7 +381,7 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
                 else:
                     setattr(record, parent_name, {field: value})
 
-            for name, value in parent_values.iteritems():
+            for name, value in parent_values.items():
                 set_parent_value(self, name, value)
             self._init_values = self._values.copy()
         else:
@@ -396,7 +405,7 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
         return '%s,%s' % (self.__name__, self.id)
 
     def __unicode__(self):
-        return u'%s,%s' % (self.__name__, self.id)
+        return '%s,%s' % (self.__name__, self.id)
 
     def __repr__(self):
         if self.id is None or self.id < 0:
@@ -423,7 +432,7 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
     def __hash__(self):
         return hash((self.__name__, self.id))
 
-    def __nonzero__(self):
+    def __bool__(self):
         return True
 
     @property
@@ -437,7 +446,7 @@ class Model(WarningErrorMixin, URLMixin, PoolBase):
         """
         values = {}
         if self._values:
-            for fname, value in self._values.iteritems():
+            for fname, value in self._values.items():
                 field = self._fields[fname]
                 if field._type in ('many2one', 'one2one', 'reference'):
                     if value:
