@@ -60,14 +60,26 @@ class Request(_Request):
         auth = self.authorization
         if not auth:
             return None
+        context = {'_request': self.context}
         if auth.type == 'session':
             user_id = security.check(
-                database_name, auth.get('userid'), auth.get('session'))
+                database_name, auth.get('userid'), auth.get('session'),
+                context=context)
         else:
             user_id = security.login(
-                database_name, auth.username, auth, cache=False)
+                database_name, auth.username, auth, cache=False,
+                context=context)
         return user_id
 
+    @cached_property
+    def context(self):
+        return {
+            'remote_addr': self.remote_addr,
+            'http_host': self.environ.get('HTTP_HOST'),
+            'scheme': self.scheme,
+            'is_secure': self.is_secure,
+            }
+
 
 def parse_authorization_header(value):
     if not value:
@@ -122,10 +134,11 @@ def with_transaction(readonly=None):
                     readonly_ = False
                 else:
                     readonly_ = True
+            context = {'_request': request.context}
             for count in range(config.getint('database', 'retry'), -1, -1):
                 with Transaction().start(
-                        pool.database_name, 0, readonly=readonly_
-                        ) as transaction:
+                        pool.database_name, 0, readonly=readonly_,
+                        context=context) as transaction:
                     try:
                         result = func(request, pool, *args, **kwargs)
                     except DatabaseOperationalError:
@@ -171,7 +184,7 @@ def user_application(name, json=True):
                     transaction.set_context(_check_access=True):
                 try:
                     response = func(request, *args, **kwargs)
-                except Exception, e:
+                except Exception as e:
                     if isinstance(e, HTTPException):
                         raise
                     logger.error('%s', request, exc_info=True)
